#!/usr/bin/env python

"""
Handlers for a resource.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.data import get_address, to_part
from imiptools.handlers import Handler
from imiptools.handlers.common import CommonFreebusy, Outgoing

class ResourceHandler(Handler, Outgoing):

    "Handling mechanisms specific to resources."

    def _record_and_respond(self, handle_for_attendee):

        """
        Record details from the incoming message, using the given
        'handle_for_attendee' callable to process any valid message
        appropriately.
        """

        oa = self.require_organiser_and_attendees()
        if not oa:
            return None

        organiser_item, attendees = oa

        # Process each attendee separately.

        calendar = []

        # Process for the current user, a resource as attendee.

        if not self.have_new_object() or not self.is_attendee(self.user):
            return None

        # Collect response objects produced when handling the request.

        response = handle_for_attendee()
        if response:
            calendar.append(response)

        return calendar

    def _schedule_for_attendee(self):

        """
        Schedule the current object for the current user.
        """

        # Interpretation of periods can depend on the time zone.

        tzid = self.get_tzid()

        # If newer than any old version, discard old details from the
        # free/busy record and check for suitability.

        periods = self.obj.get_periods(tzid, self.get_window_end())
        freebusy = self.store.get_freebusy(self.user)
        scheduled = self.can_schedule(freebusy, periods)

        # Update the participation of the resource in the object.

        attendee_attr = self.update_participation(self.obj,
            scheduled and "ACCEPTED" or "DECLINED")

        # Set the complete event or an additional occurrence.

        event = self.obj.to_node()
        self.store.set_event(self.user, self.uid, self.recurrenceid, event)

        # Remove additional recurrences if handling a complete event.

        if not self.recurrenceid:
            self.store.remove_recurrences(self.user, self.uid)

        # Update free/busy information.

        self.update_event_in_freebusy(from_organiser=False)

        # Make a version of the object with just this attendee, update the
        # DTSTAMP in the response, and return the object for sending.

        self.obj["ATTENDEE"] = [(self.user, attendee_attr)]
        self.update_dtstamp()
        return event

    def _cancel_for_attendee(self):

        """
        Cancel for the current user their attendance of the event described by
        the current object.
        """

        self.store.set_event(self.user, self.uid, self.recurrenceid, self.obj.to_node())
        self.store.cancel_event(self.user, self.uid, self.recurrenceid)

        # Update free/busy information.

        self.remove_event_from_freebusy()

        # No response is required.

        return None

class Event(ResourceHandler):

    "An event handler."

    def add(self):
        pass

    def cancel(self):

        "Cancel attendance for attendees."

        self._record_and_respond(self._cancel_for_attendee)

    def counter(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    def declinecounter(self):

        """
        Since this handler does not send counter proposals, it will not handle
        replies to such proposals.
        """

        pass

    def publish(self):
        pass

    def refresh(self):
        pass

    def reply(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    def request(self):

        """
        Respond to a request by preparing a reply containing accept/decline
        information for the recipient.

        No support for countering requests is implemented.
        """

        response = self._record_and_respond(self._schedule_for_attendee)
        if response:
            self.add_result("REPLY", map(get_address, self.obj.get_values("ORGANIZER")), to_part("REPLY", response))

class Freebusy(Handler, CommonFreebusy):

    "A free/busy handler."

    def publish(self):
        pass

    def reply(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    # request provided by CommonFreeBusy.request

# Handler registry.

handlers = [
    ("VFREEBUSY",   Freebusy),
    ("VEVENT",      Event),
    ]

# vim: tabstop=4 expandtab shiftwidth=4
